-- Entry Types --
local VARIABLE = 1
local SUBMENU = 2
local FUNC = 3

-- Entry Subtypes --
local COLOR = 1
local BOOLEAN = 2

-- Booleaan to number --
local function booltonum(bool)
	local num
	if bool == true
		num = 1
	else
		num = 0
	end
	return num
end

-- On and off thingy --
local onoff = {[0] = "Off",[1] = "On"}

-- The Menu --
CMS.Menu = {
	[1] = {
		header = "Main Menu",
		contents = {
			{type = SUBMENU, var = 2, name = "General Options", desc = "Goes to General Options."},
			{type = SUBMENU, var = 3, name = "Misc. Options", desc = "Goes to Misc. Options."}
		}
	},
	
	[2] = {
		header = "General Options",
		contents = {
			{type = VARIABLE, subtype = COLOR, name = "Flash Color", varname = "flashcolor", display = R_GetNameByColor(CMS.Check("flashcolor")), minval = 1, maxval = #skincolors-1, desc = "Changes Dashmode Flashing colors.", value = CMS.Variables["flashcolor"].value},
			{type = VARIABLE, subtype = COLOR, name = "Jet Color", varname = "jetcolor", display = R_GetNameByColor(CMS.Check("jetcolor")), minval = 1, maxval = #skincolors-1, desc = "Changes the color of jets.", value = CMS.Variables["jetcolor"].value},
			{type = VARIABLE, fixedset = {"Cobalt", "Crimson", "Green", "BlackHole"}, name = "Super Color", varname = "supercolor", display = "Cobalt", maxval = 3, desc = "Changes the Super Color.", value = CMS.Variables["supercolor"].value},
			{type = VARIABLE, subtype = BOOLEAN, name = "Charge Button Switch", varname = "abilswitch", desc = "Controls wether Jump and Spin are swapped\nfor charging.", value = CMS.Variables["abilswitch"].value},
			{type = VARIABLE, fixedset = {"Custom1", "Custom2"}, varname = "switchbutton", name = "Flap/Hover Switch Button", maxval = 1, desc = "Changes the button to switch\nbetween Metal Flap and Hover.", value = CMS.Variables["switchbutton"].value},
			{type = VARIABLE, subtype = BOOLEAN, name = "Sparkles", varname = "sparkles", desc = "Toggles wether sparkles should\ndisplay when playing as Cosmetal.\nGood for lower-end devices.", value = CMS.Variables["sparkles"].value},
			{type = VARIABLE, subtype = BOOLEAN, name = "Aim Behaviour", varname = "newaim", desc = "Changes how aiming in Dashmode\nworks. Pre-2.0 slows you down,\nand 2.0+ instantly stops you, allowing for better control.", fixedset = {"Pre-2.0", "2.0+"}, value = CMS.Variables["newaim"].value},
			{type = VARIABLE, name = "Super Music", varname = "supermusic", desc = "Changes the music for Cosmetal's Super Form", fixedset = {"Default","Vs Metal Sonic","Metal Sonic Boss 2"}, maxval = 2, value = CMS.Variables["supermusic"].value}
		}
	},
	
	[3] = {
		header = "Misc. Options",
		contents = {
			{type = VARIABLE, subtype = COLOR, default = SKINCOLOR_METALLIX, name = "Bar Color", varname = "barcolor", display = R_GetNameByColor(CMS.Check("barcolor")), minval = 1, maxval = #skincolors-1, desc = "Changes the color of ability cooldown bars.", value = CMS.Variables["barcolor"].value},
			{type = VARIABLE, subtype = BOOLEAN, name = "Subtitles", varname = "subtitles", desc = "Changes wether subtitles are enabled for Cosmetal's voice lines.", value = CMS.Variables["subtitles"].value},
			{type = VARIABLE, subtype = BOOLEAN, name = "Memes", varname = "memes", desc = "Changes wether memes happen in certain situations.", value = CMS.Variables["memes"].value},
			{type = VARIABLE, fixedset = {"VSMETL", "BCZ1", "LGRACE"}, name = "DJ Music", varname = "metaltunes", display = CMS.tunes[metaltunes], desc = "Changes the music played when you press TossFlag.", value = CMS.Variables["metaltunes"].value, maxval = 2},
			{type = VARIABLE, subtype = BOOLEAN, name = "Emeralds HUD", varname = "emeraldshud", desc = "Changes wether Chaos (and/or Sol) Emeralds display in the HUD.", value = CMS.Variables["emeraldshud"].value},
			{type = VARIABLE, subtype = BOOLEAN, name = "Lives HUD", varname = "liveshud", desc = "Changes wether to use the Lives HUD introduced in v1.5.", value = CMS.Variables["liveshud"].value},
			{type = VARIABLE, subtype = BOOLEAN, name = "Hyper Music Override", varname = "hyperoverride", desc = "Changes wether to use Cosmetal's Super music as Hyper Music or not.", value = CMS.Variables["hyperoverride"].value},
			{type = VARIABLE, subtype = BOOLEAN, name = "Menu Music", varname = "menumusic", desc = "Changes wether or not to play\nmusic while in this menu.", value = CMS.Variables["menumusic"].value},
			{type = VARIABLE, subtype = BOOLEAN, name = "Attracting Rings", varname = "mobjthinkers", desc = "Toggles wether or not rings attract to Golden Cosmetal.\nGood to switch off in big levels.\n\128ADMIN ONLY.", admin = true, value = CMS.Variables["mobjthinkers"].value}
		}
	}
}

-- Setup minval for entries that don't have one --
for i in ipairs(CMS.Menu) do
	for _,entry in ipairs(CMS.Menu[i].contents) do
		if entry.minval == nil and entry.subtype != BOOLEAN then entry.minval = 0 end
	end
end

-- The Thinker --
function CMS.MenuThinker(p, menu)
	p.pflags = $|PF_FULLSTASIS -- Stop right there, criminal scum!
	
	-- Define the entry we're on
	local entry = CMS.Menu[menu.page].contents[menu.entry]
	
	-- Moving up and down, as well as changing values --
	if menu.movecooldown == 0
		if p.cmd.forwardmove > 0
			if not menu.movedown
				if menu.entry > 1
					menu.entry = $-1
				else
					menu.entry = #CMS.Menu[menu.page].contents
				end
				menu.movecooldown = 6
			end
			menu.movedown = true
			S_StartSound(nil, sfx_menu1, p)
		elseif p.cmd.forwardmove < 0
			if not menu.movedown
				if menu.entry < #CMS.Menu[menu.page].contents
					menu.entry = $+1
				else
					menu.entry = 1
				end
				menu.movecooldown = 6
			end
			menu.movedown = true
			S_StartSound(nil, sfx_menu1, p)
		else
			menu.movedown = false
		end
		if p.cmd.sidemove > 0
			if not menu.movedown and entry.type != SUBMENU and ((entry.admin and (IsPlayerAdmin(p) or consoleplayer == server)) or not entry.admin)
				if entry.value != nil
					if entry.subtype != BOOLEAN
						if entry.value < entry.maxval
							entry.value = $+1
						else
							entry.value = entry.minval
						end
					else
						if entry.value
							entry.value = false
						else
							entry.value = true
						end
					end
					if not entry.table
						CMS.Set(entry.varname, entry.value)
					else
						entry.table[entry.varname].value = entry.value
					end
					menu.movecooldown = 6
				end
			end
			menu.movedown = true
			if entry.type != SUBMENU
				S_StartSound(nil, sfx_menu1, p)
			end
		elseif p.cmd.sidemove < 0
			if not menu.movedown and entry.type != SUBMENU and ((entry.admin and (IsPlayerAdmin(player) or consoleplayer == server)) or not entry.admin)
				if entry.value != nil
					if entry.subtype != BOOLEAN
						if entry.value > entry.minval
							entry.value = $-1
						else
							entry.value = entry.maxval
						end
					else
						if entry.value
							entry.value = false
						else
							entry.value = true
						end
					end
					if entry.table == nil
						CMS.Set(entry.varname, entry.value)
					else
						entry.table[entry.varname].value = entry.value
					end
					menu.movecooldown = 6
				end
			end
			menu.movedown = true
			if entry.type != SUBMENU
				S_StartSound(nil, sfx_menu1, p)
			end
		else
			menu.movedown = false
		end
	else
		menu.movecooldown = $-1
	end
	
	-- Jump and Spin now --
	if p.cmd.buttons&BT_JUMP
		if not menu.jumpdown
			if entry.enabledby != nil and not _G[entry.enabledby]
				S_StartSound(nil, sfx_lose, p)
			else
				if entry.type == SUBMENU
					menu.page = entry.var
					menu.entry = 1
					S_StartSound(nil, sfx_menu1, p)
				elseif entry.type == FUNC
					if entry.func
						entry.func(menu.entry)
					end
					if entry.sound
						S_StartSound(nil, entry.sound, p)
					end
				end
			end
		end
		menu.jumpdown = true
	else
		menu.jumpdown = false
	end
	if p.cmd.buttons&BT_SPIN
		if not menu.spindown
			if not CMS.Menu[menu.page].prev or CMS.Menu[menu.page].prev and CMS.Menu[menu.page].prev != -1
				if not CMS.Menu[menu.page].prev
					if menu.page > 1
						menu.page = 1
						menu.entry = 1
					else
						for i in ipairs(CMS.Menu) do
							for _,entry in ipairs(CMS.Menu[i]) do
								if entry.varname != nil
									CMS.Set(entry.varname, entry.value)
								end
							end
						end
						CONS_Printf(p, "\131Settings saved for Cosmetal!")
						P_RestoreMusic(p)
						menu.opened = false
						menu.page = 1
						menu.entry = 1
					end
				else
					menu.page = CMS.Menu[menu.page].prev
					menu.entry = 1
				end
			end
		end
		menu.spindown = true
	else
		menu.spindown = false
	end
	
	menu.iconscale = max(FRACUNIT/2, $-FRACUNIT/2)
	menu.iconscale2 = max(FRACUNIT, $-FRACUNIT/2)
	menu.timer = $+1
end

-- Open na noor --
local function openMenu(p)
	if (p.mo and p.mo.valid)
		if not p.cmsMenu.opened
			p.cmsMenu.opened = true
			if CMS.Check("menumusic")
				P_PlayJingleMusic(p, "SPEC8", 0, true, JT_OTHER)
			end
			p.cmsMenu.iconscale = FRACUNIT*5
			p.cmsMenu.iconscale2 = FRACUNIT*8
		else
			for i in ipairs(CMS.Menu) do
				for _,entry in ipairs(CMS.Menu[i]) do
					if entry.varname != nil
						CMS.Set(entry.varname, entry.value)
					end
				end
			end
			CONS_Printf(p, "\131Settings saved for Cosmetal!")
			if CMS.Check("menumusic")
				P_RestoreMusic(p)
			end
			p.cmsMenu.opened = false
		end
	else
		CONS_Printf(p, "You need to be in a level to use this.")
	end
end

addHook("PreThinkFrame", function()
	for p in players.iterate do
		if not p.cmsMenu then
			p.cmsMenu = {
				opened = false,
				movedown = false,
				jumpdown = false,
				spindown = false,
				movecooldown = 0,
				entry = 1,
				page = 1,
				iconscale = 0,
				iconscale2 = 0,
				timer = 0
			}
		end
		for i in ipairs(CMS.Menu) do
			for _,entry in ipairs(CMS.Menu[i].contents) do
				if entry.value != nil
					if entry.subtype == COLOR then entry.display = R_GetNameByColor(entry.value) end
					if entry.fixedset and type(entry.value) != "boolean" then entry.display = entry.fixedset[entry.value+1] end
					if entry.subtype == BOOLEAN then
						if entry.fixedset
							entry.display = entry.fixedset[booltonum(entry.value)+1]
						else
							entry.display = onoff[booltonum(entry.value)]
						end
					end
				end
			end
		end
		if p.cmd.buttons&BT_FIRENORMAL
			if not G_RingSlingerGametype()
				if not p.firedown
					openMenu(p)
				end
			end
			p.firedown = true
		else
			p.firedown = false
		end
		-- Closen na noor --
		if p.cmsMenu.opened then
			if p.exiting then
				p.cmsMenu.opened = false
			end
			CMS.MenuThinker(p, p.cmsMenu)
		end
	end
end)

-- Menu HUD Time boi --
hud.add(function(v, p)
	if not (p.cmsMenu and p.cmsMenu.opened) then return end
	local menu = p.cmsMenu
	local flags = V_PERPLAYER|V_HUDTRANS
	local strflags = flags|V_ALLOWLOWERCASE

	local zigzag = v.cachePatch("LTZIGZAG")
	local zigtext = v.cachePatch("CMSTEXT")
	local height = zigzag.height
	local height2 = zigtext.height
	local time = menu.timer%height

	-- Little snippet of ChrispyChars menu code, made by Chrispy --
	local BLACK = v.cachePatch("~030")
	local splitplayer = splitscreen and p == secondarydisplayplayer
	v.drawStretched(0, splitplayer and (v.height()*FRACUNIT >> 1) or 0,
		FixedDiv(v.width()*FRACUNIT, BLACK.width*FRACUNIT),
		FixedDiv(v.height()*FRACUNIT, BLACK.height*FRACUNIT << (splitscreen and 1 or 0)),
		BLACK, V_30TRANS|V_NOSCALEPATCH)
	-- Back to my code --
	v.draw(0,-time,zigzag,flags|V_SNAPTOLEFT)
	v.draw(0,-time+height,zigzag,flags|V_SNAPTOLEFT)
	v.draw(0,time,zigtext,flags|V_SNAPTOLEFT)
	v.draw(0,time-height2,zigtext,flags|V_SNAPTOLEFT)
	v.drawScaled(270*FRACUNIT, 130*FRACUNIT + 4*sin(FixedAngle(menu.timer*FRACUNIT)), menu.iconscale, v.cachePatch("ENDEGRK0"), flags)

	v.drawString(160, 20, CMS.Menu[menu.page].title or "Cosmetal Menu", strflags|V_BLUEMAP, "center")
	v.drawFill(130, splitplayer and v.height()+29 or 29, 60, 1, 73)
	v.drawFill(190, splitplayer and v.height()+29 or 29, 1, 1, 31)
	v.drawFill(131, splitplayer and v.height()+30 or 30, 60, 1, 31)

	v.drawString(160, 32, CMS.Menu[menu.page].header, strflags, "thin-center")
	for i,entry in ipairs(CMS.Menu[p.cmsMenu.page].contents) do
		local disabled = (entry.enabledby != nil and not _G[entry.enabledby]) and V_REDMAP or 0
		local active = not disabled and menu.entry == i and V_YELLOWMAP or 0

		local text
		if not entry.display
			text = tostring(entry.value)
		else
			text = entry.display
		end
		if entry.type == SUBMENU then text = "" end

		local align
		if entry.align
			align = entry.align
		else
			align = "center"
		end
		local nameandvalue
		if entry.type == VARIABLE
			nameandvalue = entry.name .. ": " .. text
		else
			nameandvalue = entry.name
		end
		
		v.drawString(160, 50+(10*(i-1)), nameandvalue, strflags|active|disabled, align)
		if menu.entry == i
			v.drawString(40, 160, entry.desc, strflags, "thin")
			v.draw(160 - v.stringWidth(nameandvalue, strflags)*3/4, 50+(10*(i-1)), v.cachePatch("M_CURSOR"), flags)
		end
	end
end,"game")

-- Music --
addHook("ShouldJingleContinue", function(p, musname)
	if musname == "SPEC8" and p.cmsMenu.opened == true and CMS.Check("menumusic") then return true end
end)

COM_AddCommand("cms_menu2", openMenu, 2)
COM_AddCommand("cms_menu", openMenu)